; File: dskd.asm -> basic monitor kernel program for 320C26 /  10. Mar. 93
;
; Author: Manfred Christ / Texas Instruments
;
; Requires coff assembler version 6.40 to assemble!
;
ESC      .set     01bh        ; escape
         .width   100
         .length   55
         .title "TMS320C26 Debugger Kernal loaded via ROM from RS232"
         .version 25
         .mmregs
VERSION  .set 0100h    ; -> 1.00 version
;
         .def unused,tmp1,PREGH,PREGL,ST0,ST1,BITLEN,TIMER
         .def startadd,scratch,TREG,STACK,AUXREG,ACCUH
         .def ACCUL,length,STATUS,INTER
         .def main,start,execmd,lddm,ldpm,ldld,ldlp,exec
         .def getdata,special,address,xmtwrd,xmtbyte
         .def TIMER,len,reset,sw_trap
;
;        .bss section: be careful with some predefined addresses !!!!!
;
         .bss ACCUL,1         ; add=60h
         .bss ACCUH,1         ; add=61h
         .bss PREGL,1         ; add=62h
         .bss PREGH,1         ; add=63h
         .bss ST0,1           ; add=64h
         .bss ST1,1           ; add=65h
         .bss TREG,1          ; add=66h -> Romcode uses TREG as Bitlen!!!!!!
         .bss STACK,8         ; add=67h
         .bss AUXREG,8        ; add=6fh
         .bss TIMER,1         ; add=77h
         .bss unused,1        ; add=78h ---> not used from dskd.asm
         .bss BITLEN,1        ; add=79h \ -> used for Baudrate
         .bss scratch,1       ; add=7ah  \
         .bss tmp1,1          ; add=7bh   \
         .bss startadd,1      ; add=7ch    \ reserved for debugger kernal
         .bss length,1        ; add=7dh    /
         .bss STATUS,1        ; add=7eh   /
         .bss INTER,1         ; add=7fh -/
;----------------------------------------------------------------------
; VECTORS:  initializes vector table
;----------------------------------------------------------------------
VECTORS  .macro
         .text
reset    b    start
int0x    eint
         ret
int1x    eint
         ret
int2x    b    halt
int3x    eint
         ret
int4x    eint
         ret
int5x    eint
         ret
sw_trap  DMPREG               ; save all register in B2 on Page 0
         b    XMT_ESC         ; send ESC to synchronize DSP and PC
         .endm
;----------------------------------------------------------------------
; 'INIT26':  initializes and sends a ESC for recognition!!!!
;----------------------------------------------------------------------
INIT26   .macro
         lacc  TREG,15        ; address 0066h in B2
         add   TREG,16        ;
         sach  BITLEN         ; lenght in RAM is 1.5 * len(ROM)
XMT_ESC  lack  ESC            ; force 8 bit constant
         call  xmtbyte,ar0
         .endm
;----------------------------------------------------------------------
; GETCMD: receive 'get' command -> returns 'command' in ar0 and arp=ar0
;----------------------------------------------------------------------
GETCMD  .macro
         call  reads          ; receive a byte --> command
         sacl  scratch        ; save it for retransmit
         call  xmtbyte,ar0    ; send back program end address,returns arp=0
         lar   ar0,scratch    ; return cmd in ar0
         .endm
;----------------------------------------------------------------------
; DPM: Display Program Memory: transmit data block from C26 to HOST
;----------------------------------------------------------------------
DPM      .macro
         call address,ar0     ; returns ar6=startadd,ar7=length,arp->ar6
loop?    lacc startadd        ; start add
         tblr tmp1            ; get 'n.' word from pgm to data
         add  #1              ; increment pointer: accu++
         sacl startadd        ; save pointer (accu)
         lacc tmp1            ; prepare for transmit
         call xmtwrd          ; transmit value in ACCU to PC
         larp ar7             ; arp=ar7
         banz loop?,*-        ; repeat for all words
         ret
         .endm
;----------------------------------------------------------------------
; DDM: Display (dump) Data Memory: transfer several words from DSP to PC
;----------------------------------------------------------------------
DDM      .macro
         call address,ar0     ; returns ar6=startadd,ar7=length,arp->ar6
loop?    lacc *+
         call xmtwrd          ; transmit value in ACCU to PC
         larp ar7
         banz loop?,*-,ar6
         ret
         .endm
;----------------------------------------------------------------------
; DLD: download data memory (from PC to 320C26)
;----------------------------------------------------------------------
DLD      .macro
         call address,ar0     ; returns ar6=startadd,ar7=length,arp->ar6
loop?    call getdata
         larp ar6
         sacl *+,ar7
         banz loop?,*-
         ret
         .endm
;----------------------------------------------------------------------
; DLP: download program memory (from PC to 320C26)
;----------------------------------------------------------------------
DLP      .macro
         call address,ar0     ; returns ar6=startadd,ar7=length,arp->ar6
loop?    call getdata,ar0     ; received word is in 'tmp1'
         lacc startadd
         tblw tmp1
         add  #1              ; akku++
         sacl startadd
         larp ar7
         banz loop?,*-
         ret
         .endm
;----------------------------------------------------------------------
; DMPREG: dump register: context save of all registers to internal RAM B2
;----------------------------------------------------------------------
DMPREG   .macro
         sst0   ST0           ; save status register 0
         dint                 ; avoid double interrupts!!!!
         ldp    #ST0          ; load data page pointer to page 0
sw_trapx sacl   ACCUL         ; save accu low
         sst1   ST1           ; save status register 1
         sach   ACCUH         ; save accu high
         lacc   tim           ; load actual content of timer register
         sacl   TIMER         ; copy timer to B2
         sph    PREGH         ; save product register high
         spl    PREGL         ; save product register low
         mpy    #1            ; treg -> preg
         spl    TREG          ; save T-Register
         sar    ar0,AUXREG    ; save ar0
         sar    ar1,AUXREG+1  ; save ar1
         sar    ar2,AUXREG+2  ; save ar2
         sar    ar3,AUXREG+3  ; save ar3
         sar    ar4,AUXREG+4  ; save ar4
         sar    ar5,AUXREG+5  ; save ar5
         sar    ar6,AUXREG+6  ; save ar6
         sar    ar7,AUXREG+7  ; save ar7
         larp   ar0           ; arp=ar0
         lark   ar0,STACK     ; ar0-> stack / force 8 bit constant
         rpt    #7            ; save 8 stack register
         popd   *+            ; stack -> memory
;
; the following is necessary to rebuilt the ARP for the display!
; while keeping all the other bits unchanged (especially Data page pointer)!
;
         lst    ST0
         ldp    #ST0          ; go back to page 0 !!!! added: 14.7.92
         lst1   ST1
         ldp    ST0           ; added to correct DP problem: 14.7.92  
         sst    ST0           
         ldp    #ST0          ; added to correct DP problem: 14.7.92
         .endm
;----------------------------------------------------------------------
;  EXECUTE:   execute a program from address !
;----------------------------------------------------------------------
EXECUTE  .macro
         call   getdata,ar0   ; received word is in accu and 'tmp1'-->startadd
         sacl   STACK         ; startadd= address for restart of user program
         bioz   $+4     ; wait for start code
         b      $-2
         bioz   $
; the following lines execute a dummy interrupt to reset IF(int2) !!!!!
         lack   1             ; flag into 'scratch' for 'halt'
         blkd  ST0,tmp1       ; save B2(ST0) to temp
         eint                 ; execute the interrupt service routine
         sacl   scratch       ; set a flag for executing dummy interrupts
         blkd   tmp1,ST0      ; restore ST0 to B2(ST0)
; IFR(int2) is turned off!
         lack   0             ; reset flag
         sacl   scratch       ; in scratch!
; finished dummy interrupt!!!!!!!
         lark   ar0,STACK+7   ; force 8 bit constant
         rpt    #7            ; reload stack (6 values)
         pshd   *-
         lar    ar0,AUXREG    
         lar    ar1,AUXREG+1
         lar    ar2,AUXREG+2
         lar    ar3,AUXREG+3
         lar    ar4,AUXREG+4
         lar    ar5,AUXREG+5
         lar    ar6,AUXREG+6
         lar    ar7,AUXREG+7
         lt     PREGL
         mpy    #1
         lph    PREGH
         lt     TREG
         lacc   TIMER
         sacl   tim
         zalh   ACCUH
         adds   ACCUL
         lst1   ST1
         lst0   ST0
         eint           ; added to allow halt with INT2: 0.54
         ret
         .endm
;----------------------------------------------------------------------
;  TRANSMIT: transmit word
;----------------------------------------------------------------------
TRANSMIT .macro
xmtwrd   sacl scratch
         rptk 7
         sfr
         call _xmtbyte,ar0     ; send highbyte
         lacc scratch         ;
_xmtbyte bioz $+4             ; expect a sync (0) signal from PC
         b    $-2
; xmtbyte follows now         ; send lowbyte and return to calling pgm
;
;  'xmtbyte' requires ARP->AR0 and sets ACCU='mask for byte'
;  send startbit (0) +  databyte + stopbits (2)
;
xmtbyte: rc                   ; startbit=0
         lark ar1,#8          ; counter: 1 startbit+ 8 databits (+ 2 stopbits)
nextbit1 lar  ar0,BITLEN
         bnc  snd0,*-         ; if c=1 send 1 else send 0
snd1     sxf                  ; send one
         b    snd,*-
snd0     rxf
snd      banz snd,*-          ; send one bit
;
         ror                  ; lsb(accu) -> carrybit
         larp ar1
         banz nextbit1,*-,ar0 ; repeat for entire word (10 bits)
;    
         sxf
         ret
         .endm
;----------------------------------------------------------------------
; GETADD:   receive startaddress and length
;           return: ar6=startadd / ar7=len / arp= ar6
;----------------------------------------------------------------------
GETADD   .macro
address  call   getdata,ar0   ; receive start address and send it back!!!!
         sacl   startadd
         lar    ar6,startadd  ; len   add
         call   getdata,ar0   ; receive length and send it back!!!!
         sacl   length        ; length includes the zero!!!
         lar    ar7,length    ; len   add
         larp   ar6
         ret
         .endm
;----------------------------------------------------------------------
; GETDATA: receive a word and send it back
;          ACCU and tmp1 returns received word
;          uses 'tmp1'
;----------------------------------------------------------------------
GETDATA  .macro
getdata  call   reads         ; receive low byte of word
         sacl   tmp1          ; save low byte
         call   xmtbyte,ar0   ; send it back
         call   reads         ; receive high byte of word
         add    tmp1,8
         sacl   tmp1
         call   xmtbyte,ar0   ; send it back
         lacc   tmp1
         ret
         .endm
;----------------------------------------------------------------------
; Exchange program memory address
; 1. receive pgm address
; 2. read    content address and send content to PC
; 3. receive new content and store it
;----------------------------------------------------------------------
XCH_PGM  .macro
         call address,ar0     ; returns ar6=startadd,ar7=length,arp->ar6
         lacc startadd        ; startadd==address
         tblr startadd        ; startadd:=original content of address
         tblw length          ; length== new content
         lacc startadd
         b    xmtwrd          ; transmit value in ACCU to PC
         .endm
;----------------------------------------------------------------------
; the following macro is a software halt (like a NMI) to halt (stop) the
; 320C26 while in runmode
;----------------------------------------------------------------------
HALT     .macro
halt:    sst0   ST0           ; save status register 0
         ldp    #ST0          ; load data page pointer to page 0
         bit    scratch,15    ; TC=1 => dummy interrupt
         bbz    sw_trapx      ; if bit==0 it is a HALT => go to trap routine
         ret                  ; return from dummy interrupt
         .endm
;----------------------------------------------------------------------
; READS: read serial mode
;----------------------------------------------------------------------
READS     .macro
reads:    BIOZ   STOK,*,AR1   ;wait for start bit
          B      reads        ;
STOK      lac    BITLEN,6     ;BITLEN is scaled and
half_len  subk   171          ;decremented by 8/3 for
	  bgz	 half_len     ;BITLEN/2 wait
          zac
	  ;--------------------------------------------
	  LARK	 AR3,7	      ;number of bits - 1
WTBIT     LAR    AR1,BITLEN   ;wait for a bit
          sfr
	  BANZ	 $,*-	      ;wait for a bit
          BIOZ   ZEROBT,*,AR3 ;test bit value = 0 ?
          addk   80h          ;add bit value
ZEROBT    BANZ   WTBIT,*-,AR1 ;last bit ?
	  RET		      ;ACC = read value
          .endm
;======================================================================
;
;  BEGIN OF MAIN PROGRAM
;
;======================================================================
         VECTORS               ; macro: reserved area for user vectors
         .word special
         .word len,VERSION     ; special addresses: at init (len,version)
;                              ;                    can be used for 2 INSTR.
start:   INIT26                ; macro to initialize
main     call   execmd         ; 'main control loop
         b      main           ; execmd returns here after command execution
;
; execute commands
;
execmd   GETCMD               ; macro returns command in ar0 and arp=ar0
lddm     banz   ldpm,*-       ; dump data memory
         DDM                  ; macro 'dump data memory'    DSP -> PC
ldpm     banz   ldld,*-       ; dump program memory
         DPM                  ; macro 'dump program memory' DSP -> PC
ldld     banz   ldlp,*-       ; load data memory to c26
         DLD                  ; macro 'download data'       PC  -> DSP
ldlp     banz   xpgm,*-       ; load program memory to c26
         DLP                  ; macro 'download program'    PC  -> DSP
xpgm     banz   exec,*-       ; load pgm  word to PC
         XCH_PGM              ; macro 'exchange pgm memory word'
exec     banz   special,*-    ; execute to breakpoint or single step
         EXECUTE              ; macro 'execute program'
         GETDATA              ; macro 'get data word'
         GETADD               ; macro 'get 2 data words'
         TRANSMIT             ; macro 'transmit a word or a byte'
         READS                ; macro 'serial RS232 mode from BIO-PIN'
special  ret                  ; branch back if invalid 'code'
         HALT                 ; Interrupt (2) service routine
                              ; HALT gets overwritten from some copy/fill progs
len      .set    $-0fa00h     ; length of monitor kernal
         .end
