 ;-----------------------------------------------------------;
 ; Written By: Keith Larson                                  ;
 ;             TMS320Cxx DSP Applications Engineer           ;
 ;             Texas Instruments                             ;
 ;-----------------------------------------------------------;
 ;     CMD                  CMD 4
 ;     LENGTH LO            CALL LO
 ;     LENGTH HI            CALL HI
 ;     ADDR   LO
 ;     ADDR   HI
 ;     DATA   LO
 ;     DATA   HI
 ;
YES     .set    1
NO      .set    0
KL      .set    1
 ;-----------------------------------------------------------;
TEMP            .set   0x60     ;
WORD16          .set   0x61     ;
PADDR           .set   0x62     ;
WORD            .set   0x63     ;
TEMPx           .set   0x64     ;
SCRATCH         .set   0x65     ;
BITLEN          .set   0x66     ;RS232 bitwidth in C26 clock ticks
;----------------------------------------------------------------
        .include  "mmregs.asm"
        .ps     0xFA00          ;
        B       KLKRNL          ;FA00  RS
        B       KLKRNL          ;FA02  INT0
        B       KLKRNL          ;FA04  INT1
        B       KLKRNL          ;FA06  INT2
        B       KLKRNL          ;FA08  TINT
        ret                     ;FA0A  RINT
        nop                     ;
        ret                     ;FA0C  XINT
        nop                     ;
        B       KLKRNL          ;FA0E  TRAP
        ;--------------------------------------------------------------
        b       CMD             ;FA10  JUMP table to KLKRNL routines
	b       XMIT            ;FA12
	b       XMIT16          ;FA14
	b       RECV            ;FA16
	b       RECV16          ;FA18
	b       CXMIT           ;FA1A
        ;--------------------------------------------------------------
KLKRNL: dint                    ;
        sxf                     ;initialize line
        ldpk    0               ;
        conf    1               ;
        zac                     ;
        sacl    IMR             ;
        lac     BITLEN,1        ;2 x
        add     BITLEN          ;3 x
        sfr                     ;1.5 x
        subk    7               ;best BITLEN skew
        sacl    BITLEN          ;Store adjusted BITLEN
        sxf
        ;------------------------
CMD     call    RECV,*,AR0      ;get 8 bit CMD from host
        bz      CMD             ;
        subk    1               ;
        bz      W_PS            ;1 = write program space
        subk    1               ;
        bz      R_PS            ;2 = read program space
        subk    1               ;
        bz      W_DS            ;3 = write data space
        subk    1               ;
        bz      R_DS            ;4 = read data space
        subk    1               ;
        bnz     CMD             ;5 = Execute program at next word
        call    RECV16          ;
        lac     WORD16          ;
        bacc                    ;
;============================================================
; W_PS & R_PS are used to access program memory
;============================================================
W_PS    call    KRNL            ;
W_PS2   call    RECV16          ;copy length words to PS
        sacl    WORD            ;
        lac     PADDR           ;ACCU=src address
        tblw    WORD            ;
        addk    1               ;
        sacl    PADDR           ;
        larp    AR3             ;
        banz    W_PS2,*-,AR2    ;
        b       CMD             ;
        ;------------------------
R_PS    call    KRNL            ;
R_PS2   lac     PADDR           ;
        tblr    TEMP            ;
        addk    1               ;
        sacl    PADDR           ;
        lac     TEMP            ;
        call    XMIT16          ;
        larp    AR3             ;
        banz    R_PS2,*-,AR2    ;next word
        b       CMD             ;
;============================================================
; W_DS & R_DS are used to access data memory
;============================================================
W_DS    call    KRNL            ;
W_DS2   call    RECV16          ;build data word
        larp    AR2             ;
        sacl    *+,0,AR3        ;write the word   <<< missing shift!!!
        banz    W_DS2,*-,AR2    ;
        b       CMD             ;
        ;------------------------
R_DS    call    KRNL            ;
R_DS2   larp    AR2             ;get word
        lac     *+,0            ;
        call    XMIT16,*        ;
        larp    AR3             ;
        banz    R_DS2,*-        ;next word
        b       CMD             ;
;============================================================
; KRNL is common to all block transfers
;============================================================
KRNL    call    RECV16          ;1 st word is address
        lar     AR2,WORD16      ;
        sar     AR2,PADDR       ;
        call    RECV16          ;2 nd word is length-1
        lar     AR3,WORD16      ;
        larp    AR2             ;
        ret                     ;
;============================================================
; RECV returns word in bottom 8 bits of ACCL
;============================================================
RECV16: call    RECV            ;recv LO
        sacl    WORD16          ;
        call    RECV            ;recv HI
        rptk    7               ;
        sfl                     ;build HI+LO
        or      WORD16          ;
	sacl    WORD16          ;
	ret                     ;
	;------------------------
RECV    zac                     ;
	sfr                     ;
	lark    AR1,8           ;take 9 samples (START >> shifted out!)
	lar     AR0,BITLEN      ;
	larp    0               ;
	mar     *BR0+           ;
WAIT    bioz    NBIT,*,AR0      ;1st time wait 1/2 BITLEN extra
	b       WAIT            ;
NBIT    banz    $,*-            ;wait BITLEN
	sfr                     ;shift to new posn
	bioz    RB0,*,AR1       ;test bit
RB1     addk    0x80            ;if 1, add new bit
RB0     lar     AR0,BITLEN      ;
	banz    NBIT,*-,AR0     ;last bit?
	andk    0xff
	bioz    $               ;wait for STOP before RET
	ret                     ;ACCU = RECEIVED CHAR
;============================================================
; XMIT assumes word is in bottom 8 bits of ACCL
;============================================================
XMIT16: call    XMIT            ;xmit LO
	call    XMIT            ;xmit HI
	ret                     ;
;------------------------------------------------------------
XMIT    sacl    TEMPx           ;
	call    RECV            ;Handshake with slow host
	lac     TEMPx           ;
CXMIT   sfl                     ;first bit is 0
        lark    AR1,8           ;1 start, 8 DATA
SENDBIT ror                     ;TEMP = ........X XXXXXXXXs
        bnc     XB0,*,AR0       ;
        sxf                     ;
        b       XB1,*,AR0       ;
XB0     rxf                     ;
XB1     lar     AR0,BITLEN      ;timout BITLEN
        banz    $,*-            ;
        larp    AR1             ;
        banz    SENDBIT,*-,AR0  ;send another bit?
        sxf                     ;
        lar     AR0,BITLEN      ;
        banz    $,*-            ;
        ret                     ;
